@@ -8,10 +8,11 @@
 #include <CoreGraphics/CGDirectDisplay.h>
 #include <CoreVideo/CVDisplayLink.h>
 
+#ifdef MAC_OS_X_VERSION_10_15
 extern CFDictionaryRef CoreDisplay_DisplayCreateInfoDictionary(CGDirectDisplayID display) __attribute__((weak_import));
-#ifndef MAC_OS_X_VERSION_10_15
-#import <IOKit/graphics/IOGraphicsLib.h>
-extern CFDictionaryRef CoreDisplay_IODisplayCreateInfoDictionary(io_service_t framebuffer, IOOptionBits options)  __attribute__((weak_import));
+extern Boolean CoreDisplay_Display_IsHDRModeEnabled(CGDirectDisplayID display) __attribute__((weak_import));
+#else
+#include <IOKit/graphics/IOGraphicsLib.h>
 #endif
 
 static void detectDisplays(FFDisplayServerResult* ds)
@@ -37,36 +38,28 @@ static void detectDisplays(FFDisplayServerResult* ds)
                 {
                     const CVTime time = CVDisplayLinkGetNominalOutputVideoRefreshPeriod(link);
                     if (!(time.flags & kCVTimeIsIndefinite))
-                        refreshRate = time.timeScale / (double) time.timeValue + 0.5; //59.97...
+                        refreshRate = time.timeScale / (double) time.timeValue; //59.97...
                     CVDisplayLinkRelease(link);
                 }
             }
 
             FF_STRBUF_AUTO_DESTROY name = ffStrbufCreate();
+            CFDictionaryRef FF_CFTYPE_AUTO_RELEASE displayInfo = NULL;
             #ifdef MAC_OS_X_VERSION_10_15
             if(CoreDisplay_DisplayCreateInfoDictionary)
-            {
-                CFDictionaryRef FF_CFTYPE_AUTO_RELEASE displayInfo = CoreDisplay_DisplayCreateInfoDictionary(screen);
-                if(displayInfo)
-                {
-                    CFDictionaryRef productNames;
-                    if(!ffCfDictGetDict(displayInfo, CFSTR(kDisplayProductName), &productNames))
-                        ffCfDictGetString(productNames, CFSTR("en_US"), &name);
-                }
-            }
+                displayInfo = CoreDisplay_DisplayCreateInfoDictionary(screen);
             #else
-            if(CoreDisplay_IODisplayCreateInfoDictionary)
             {
                 io_service_t servicePort = CGDisplayIOServicePort(screen);
-                CFDictionaryRef FF_CFTYPE_AUTO_RELEASE displayInfo = CoreDisplay_IODisplayCreateInfoDictionary(servicePort, kIODisplayOnlyPreferredName);
-                if(displayInfo)
-                {
-                    CFDictionaryRef productNames;
-                    if(!ffCfDictGetDict(displayInfo, CFSTR(kDisplayProductName), &productNames))
-                        ffCfDictGetString(productNames, CFSTR("en_US"), &name);
-                }
+                displayInfo = IODisplayCreateInfoDictionary(servicePort, kIODisplayOnlyPreferredName);
             }
             #endif
+            if(displayInfo)
+            {
+                CFDictionaryRef productNames;
+                if(!ffCfDictGetDict(displayInfo, CFSTR(kDisplayProductName), &productNames))
+                    ffCfDictGetString(productNames, CFSTR("en_US"), &name);
+            }
 
             CGSize size = CGDisplayScreenSize(screen);
 
@@ -86,21 +79,20 @@ static void detectDisplays(FFDisplayServerResult* ds)
             );
             if (display)
             {
-                // Shitty code
-                uint8_t bitDepth = 0;
-                FF_CFTYPE_AUTO_RELEASE CFStringRef desc = CFCopyDescription(mode);
-                CFRange start = CFStringFind(desc, CFSTR("BitsPerSample = "), 0);
-                if (start.location != kCFNotFound)
+                // https://stackoverflow.com/a/33519316/9976392
+                // Also shitty, but better than parsing `CFCopyDescription(mode)`
+                CFDictionaryRef dict = (CFDictionaryRef) *((int64_t *)mode + 2);
+                if (CFGetTypeID(dict) == CFDictionaryGetTypeID())
                 {
-                    for (CFIndex idx = start.location + start.length; idx < CFStringGetLength(desc); ++idx)
-                    {
-                        UniChar ch = CFStringGetCharacterAtIndex(desc, idx);
-                        if (!ffCharIsDigit((char) ch))
-                            break;
-                        bitDepth = (uint8_t) (bitDepth * 10 + (ch - '0'));
-                    }
+                    int32_t bitDepth;
+                    ffCfDictGetInt(dict, kCGDisplayBitsPerSample, &bitDepth);
+                    display->bitDepth = (uint8_t) bitDepth;
+                }
+
+                if (CoreDisplay_Display_IsHDRModeEnabled)
+                {
+                    display->hdrEnabled = CoreDisplay_Display_IsHDRModeEnabled(screen);
                 }
-                display->bitDepth = bitDepth;
             }
             CGDisplayModeRelease(mode);
         }
