@@ -30,10 +30,10 @@ void ffPrintTerminalSize(FFTerminalSizeOptions* options)
         else
         {
             FF_PRINT_FORMAT_CHECKED(FF_TERMINALSIZE_DISPLAY_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_TERMINALSIZE_NUM_FORMAT_ARGS, ((FFformatarg[]){
-                {FF_FORMAT_ARG_TYPE_UINT16, &result.rows},
-                {FF_FORMAT_ARG_TYPE_UINT16, &result.columns},
-                {FF_FORMAT_ARG_TYPE_UINT16, &result.width},
-                {FF_FORMAT_ARG_TYPE_UINT16, &result.height}
+                {FF_FORMAT_ARG_TYPE_UINT16, &result.rows, "rows"},
+                {FF_FORMAT_ARG_TYPE_UINT16, &result.columns, "columns"},
+                {FF_FORMAT_ARG_TYPE_UINT16, &result.width, "width"},
+                {FF_FORMAT_ARG_TYPE_UINT16, &result.height, "height"},
             }));
         }
     }
@@ -94,10 +94,10 @@ void ffGenerateTerminalSizeJsonResult(FF_MAYBE_UNUSED FFTerminalOptions* options
 void ffPrintTerminalSizeHelpFormat(void)
 {
     FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_TERMINALSIZE_MODULE_NAME, "{1} columns x {2} rows ({3}px x {4}px)", FF_TERMINALSIZE_NUM_FORMAT_ARGS, ((const char* []) {
-        "Terminal rows",
-        "Terminal columns",
-        "Terminal width (in pixels)",
-        "Terminal height (in pixels)"
+        "Terminal rows - rows",
+        "Terminal columns - columns",
+        "Terminal width (in pixels) - width",
+        "Terminal height (in pixels) - height",
     }));
 }
 
@@ -114,7 +114,7 @@ void ffInitTerminalSizeOptions(FFTerminalSizeOptions* options)
         ffPrintTerminalSizeHelpFormat,
         ffGenerateTerminalSizeJsonConfig
     );
-    ffOptionInitModuleArg(&options->moduleArgs);
+    ffOptionInitModuleArg(&options->moduleArgs, "󰲎");
 }
 
 void ffDestroyTerminalSizeOptions(FFTerminalSizeOptions* options)
